/*
  Copyright (c) 2006-2008 Gordon Gremme <gremme@zbh.uni-hamburg.de>
  Copyright (c) 2006-2008 Center for Bioinformatics, University of Hamburg

  Permission to use, copy, modify, and distribute this software for any
  purpose with or without fee is hereby granted, provided that the above
  copyright notice and this permission notice appear in all copies.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

/* This implementation was heavily inspired by code from Stefan Kurtz. */

#include "core/assert_api.h"
#include "core/codon.h"

const char gt_aminos[] = "FFLLSSSSYY**CC*WLLLLPPPPHHQQRRRR"
                         "IIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG";

char gt_codon2amino(char n0, char n1, char n2)
{
  unsigned int code = 0;

  switch (n0) {
    case 'A':
    case 'a':
      code = GT_A_CODE << 4;
      break;
    case 'C':
    case 'c':
      code = GT_C_CODE << 4;
     break;
    case 'G':
    case 'g':
      code = GT_G_CODE << 4;
      break;
    case 'T':
    case 't':
    case 'U':
    case 'u':
      code = GT_T_CODE << 4;
      break;
    default: gt_assert(0); /* XXX */
  }

  switch (n1) {
    case 'A':
    case 'a':
      code += GT_A_CODE << 2;
      break;
    case 'C':
    case 'c':
      code += GT_C_CODE << 2;
      break;
    case 'G':
    case 'g':
      code += GT_G_CODE << 2;
      break;
    case 'T':
    case 't':
    case 'U':
    case 'u':
      code += GT_T_CODE << 2;
      break;
    default: gt_assert(0); /* XXX */
  }

  switch (n2) {
    case 'A':
    case 'a':
      code += GT_A_CODE;
      break;
    case 'C':
    case 'c':
      code += GT_C_CODE;
      break;
    case 'G':
    case 'g':
      code += GT_G_CODE;
      break;
    case 'T':
    case 't':
    case 'U':
    case 'u':
      code += GT_T_CODE;
      break;
    default: gt_assert(0); /* XXX */
  }

  return gt_aminos[code];
}
